@@ -1,3 +1,7 @@
+"""Store data streams in different backends.
+
+"""
+
 from stor import stor_output, stor_input, stor_util
 from directory_backend import directory_backend
 from combine_backend import combine_backend
